<?php

/**
 * The interface with the links database.
 */
class Links
{
    /**
     * The database instance to interact with.
     *
     * @var SQLite3
     */
    private $db;


    /**
     * Connects with the database.
     *
     * @param string $database the path to the database
     * @param string $schema the path to the database schema, in case the database does not exist
     */
    function __construct(string $database, string $schema)
    {
        if (!file_exists($database)) {
            $this->db = new SQLite3($database);
            $this->db->query(file_get_contents($schema));
        } else {
            $this->db = new SQLite3($database, SQLITE3_OPEN_READWRITE);
        }
        $this->db->enableExceptions(true);

        $this->deleteExpiredLinks();
    }

    /**
     * Destroys this instance, disconnecting from the database.
     */
    function __destruct()
    {
        $this->db->close();
    }


    /**
     * Returns the target to the given path, or null if there is no such target.
     *
     * @param string $path the path to return the target of
     * @return array|null the target to the given path and whether confirmation is required before redirecting, or null
     * if there is no such target
     */
    function getTarget(string $path): ?array
    {
        $stmt = $this->db->prepare("SELECT target, needs_confirmation FROM links WHERE path = :path;");
        $stmt->bindValue(":path", $path);
        if ($result = $stmt->execute()->fetchArray(SQLITE3_ASSOC))
            return array($result["target"], $result["needs_confirmation"]);

        return null;
    }

    /**
     * Deletes all expired links.
     */
    function deleteExpiredLinks(): void {
        $now = date("Y-m-d H:i:s");
        $stmt = $this->db->prepare("DELETE FROM links WHERE expires_at < :now;");
        $stmt->bindValue(":now", $now);
        $stmt->execute();
    }
}
